@@ -7,17 +7,18 @@ import discord
 
 from .slash_core import SlashCommandResponse, get_class
 from .slash_command import SlashCommand, SlashCommandPermissions
+from .utils import ClickListener, _on_button_click
 from ._decohub import _HANDLER
 from ._send_modifications import *
 
-from ..interactions import SlashInteraction, ButtonInteraction
+from ..interactions import ComponentType, SlashInteraction, MessageInteraction
 
 
 __all__ = ("SlashClient",)
 
 
 class SlashClient:
-    '''
+    """
     The main purpose of this class is to track ``INTERACTION_CREATE`` API event.
 
     Parameters
@@ -39,7 +40,7 @@ class SlashClient:
         All working slash commands
     is_ready : bool
         Equals to ``True`` if SlashClient is ready, otherwise it's ``False``
-    '''
+    """
     def __init__(self, client, *, show_warnings: bool=False, modify_send: bool=True):
         _HANDLER.client = client
         self.client = _HANDLER.client
@@ -70,6 +71,8 @@ class SlashClient:
             self.client.add_listener(self._on_ready, 'on_ready')
         else:
             self.client.add_listener(self._on_connect, 'on_connect')
+        # For nice click listener
+        self.client.add_listener(_on_button_click, 'on_button_click')
 
     def _modify_discord(self):
         # Modify cog loader
@@ -126,11 +129,15 @@ class SlashClient:
         
         def get_command_named(guild, name):
             return self.get_guild_command_named(guild.id, name)
+        
+        def create_click_listener(message, timeout=None):
+            return ClickListener(message.id, timeout)
 
         if self._modify_send:
             Messageable.send = send_with_components
             discord.Message.edit = edit_with_components
         Context.wait_for_button_click = ctx_wait_for_button_click
+        discord.Message.create_click_listener = create_click_listener
         discord.Message.wait_for_button_click = message_wait_for_button_click
         discord.Guild.get_commands = get_commands
         discord.Guild.get_command = get_command
@@ -170,7 +177,7 @@ class SlashClient:
         return [sc for sc in self._global_commands.values()]
 
     def event(self, func):
-        '''
+        """
         Decorator
         ::
         
@@ -181,7 +188,7 @@ class SlashClient:
         | All possible events:
         | ``on_ready``, ``on_auto_register``,
         | ``on_slash_command``, ``on_slash_command_error``
-        '''
+        """
         if not asyncio.iscoroutinefunction(func):
             raise TypeError(f'<{func.__qualname__}> must be a coroutine function')
         name = func.__name__
@@ -195,7 +202,7 @@ class SlashClient:
         return func
 
     def command(self, *args, **kwargs):
-        '''
+        """
         A decorator that registers a function below as response for specified slash-command.
 
         Parameters are similar to SlashCommand arguments.
@@ -219,7 +226,7 @@ class SlashClient:
         guild_ids : List[int]
             if specified, the client will register a command in these guilds.
             Otherwise this command will be registered globally. Requires ``description``
-        '''
+        """
         def decorator(func):
             if not asyncio.iscoroutinefunction(func):
                 raise TypeError(f'<{func.__qualname__}> must be a coroutine function')
@@ -338,20 +345,22 @@ class SlashClient:
 
     # Straight references to API
     async def fetch_global_commands(self):
-        '''Requests a list of global registered commands from the API
+        """
+        Requests a list of global registered commands from the API
 
         Returns
         -------
 
         global_commands : List[SlashCommand]
-        '''
+        """
         data = await self.client.http.request(
             Route('GET', '/applications/{app_id}/commands', app_id=self.client.user.id)
         )
         return [SlashCommand.from_dict(dat) for dat in data]
 
     async def fetch_guild_commands(self, guild_id: int):
-        '''Requests a list of registered commands for a specific guild
+        """
+        Requests a list of registered commands for a specific guild
 
         Parameters
         ----------
@@ -362,7 +371,7 @@ class SlashClient:
         -------
 
         guild_commands : List[SlashCommand]
-        '''
+        """
         data = await self.client.http.request(
             Route('GET', '/applications/{app_id}/guilds/{guild_id}/commands',
             app_id=self.client.user.id, guild_id=guild_id)
@@ -370,7 +379,8 @@ class SlashClient:
         return [SlashCommand.from_dict(dat) for dat in data]
     
     async def fetch_global_command(self, command_id: int):
-        '''Requests a registered global slash-command
+        """
+        Requests a registered global slash-command
 
         Parameters
         ----------
@@ -381,7 +391,7 @@ class SlashClient:
         -------
 
         global_command : SlashCommand
-        '''
+        """
         data = await self.client.http.request(
             Route(
                 "GET",
@@ -393,7 +403,7 @@ class SlashClient:
         return SlashCommand.from_dict(data)
 
     async def fetch_guild_command(self, guild_id: int, command_id: int):
-        '''
+        """
         Requests a registered guild command
 
         Parameters
@@ -407,7 +417,7 @@ class SlashClient:
         -------
 
         guild_command : SlashCommand
-        '''
+        """
         data = await self.client.http.request(
             Route(
                 "GET",
@@ -420,7 +430,8 @@ class SlashClient:
         return SlashCommand.from_dict(data)
 
     async def register_global_slash_command(self, slash_command: SlashCommand):
-        '''Registers a global slash-command
+        """
+        Registers a global slash-command
 
         .. seealso:: :ref:`raw_slash_command`
         
@@ -428,7 +439,7 @@ class SlashClient:
         ----------
 
         slash_command : SlashCommand
-        '''
+        """
         if not isinstance(slash_command, SlashCommand):
             raise ValueError('Expected a <SlashCommand> instance')
         r = await self.client.http.request(
@@ -444,7 +455,8 @@ class SlashClient:
         return sc
     
     async def register_guild_slash_command(self, guild_id: int, slash_command: SlashCommand):
-        '''Registers a local slash-command
+        """
+        Registers a local slash-command
         
         .. seealso:: :ref:`raw_slash_command`
         
@@ -454,7 +466,7 @@ class SlashClient:
         guild_id : int
 
         slash_command : SlashCommand
-        '''
+        """
         if not isinstance(slash_command, SlashCommand):
             raise discord.InvalidArgument('Expected a <SlashCommand> instance')
         r = await self.client.http.request(
@@ -472,14 +484,14 @@ class SlashClient:
         return sc
     
     async def overwrite_global_commands(self, slash_commands: list):
-        '''
+        """
         Bulk overwrites all global commands
         
         Parameters
         ----------
 
         slash_commands : List[SlashCommand]
-        '''
+        """
 
         if not all(isinstance(sc, SlashCommand) for sc in slash_commands):
             raise discord.InvalidArgument("slash_commands must contain only SlashCommand instances")
@@ -497,7 +509,7 @@ class SlashClient:
         return new_commands
 
     async def overwrite_guild_commands(self, guild_id: int, slash_commands: list):
-        '''
+        """
         Bulk overwrites all guild commands
         
         Parameters
@@ -506,7 +518,7 @@ class SlashClient:
         guild_id : int
 
         slash_commands : List[SlashCommand]
-        '''
+        """
         if not all(isinstance(sc, SlashCommand) for sc in slash_commands):
             raise discord.InvalidArgument("slash_commands must contain only SlashCommand instances")
         await self.client.http.request(
@@ -523,7 +535,7 @@ class SlashClient:
         self._guild_commands[guild_id] = {cmd.id: cmd for cmd in new_commands}
 
     async def edit_global_slash_command(self, command_id: int, slash_command: SlashCommand, **kwargs):
-        '''
+        """
         Edits a global command
 
         Parameters
@@ -531,7 +543,7 @@ class SlashClient:
         command_id : int
         slash_command : SlashCommand
             replacement of the old data
-        '''
+        """
         if not isinstance(slash_command, SlashCommand):
             raise discord.InvalidArgument('parameter slash_command must be SlashCommand')
         ignore_name = kwargs.get("ignore_name", False)
@@ -550,7 +562,8 @@ class SlashClient:
         return sc
     
     async def edit_guild_slash_command(self, guild_id: int, command_id: int, slash_command: SlashCommand, **kwargs):
-        '''Edits a local command
+        """
+        Edits a local command
 
         Parameters
         ----------
@@ -558,7 +571,7 @@ class SlashClient:
         command_id : int
         slash_command : SlashCommand
             replacement of the old data
-        '''
+        """
         if not isinstance(slash_command, SlashCommand):
             raise discord.InvalidArgument('parameter slash_command must be SlashCommand')
         ignore_name = kwargs.get("ignore_name", False)
@@ -578,13 +591,14 @@ class SlashClient:
         return sc
     
     async def delete_global_slash_command(self, command_id: int):
-        '''Deletes a global command
+        """
+        Deletes a global command
 
         Parameters
         ----------
 
         command_id : int
-        '''
+        """
         await self.client.http.request(
             Route(
                 "DELETE",
@@ -597,7 +611,8 @@ class SlashClient:
         self._remove_global_command(command_id)
     
     async def delete_guild_slash_command(self, guild_id: int, command_id: int):
-        '''Deletes a local command
+        """
+        Deletes a local command
 
         Parameters
         ----------
@@ -605,7 +620,7 @@ class SlashClient:
         guild_id : int
 
         command_id : int
-        '''
+        """
         await self.client.http.request(
             Route(
                 "DELETE",
@@ -743,7 +758,7 @@ class SlashClient:
 
     # Even slower API methods
     async def fetch_global_command_named(self, name: str):
-        '''
+        """
         Fetches a global command that matches the specified name
 
         Parameters
@@ -751,14 +766,14 @@ class SlashClient:
 
         name : str
             the name of the command to fetch
-        '''
+        """
         cmds = await self.fetch_global_commands()
         for c in cmds:
             if c.name == name:
                 return c
 
     async def fetch_guild_command_named(self, guild_id: int, name: str):
-        '''
+        """
         Fetches a guild command that matches the specified name
 
         Parameters
@@ -769,14 +784,14 @@ class SlashClient:
 
         name : str
             the name of the command to fetch
-        '''
+        """
         cmds = await self.fetch_guild_commands(guild_id)
         for cmd in cmds:
             if cmd.name == name:
                 return cmd
 
     async def edit_global_command_named(self, name: str, slash_command: SlashCommand):
-        '''
+        """
         Edits a global command matching the specified name.
 
         Parameters
@@ -787,13 +802,13 @@ class SlashClient:
         
         slash_command : SlashCommand
             replacement of the old data
-        '''
+        """
         cmd = self.get_global_command_named(name)
         if cmd is not None:
             await self.edit_global_command(cmd.id, slash_command)
 
     async def edit_guild_command_named(self, guild_id: int, name: str, slash_command: SlashCommand):
-        '''
+        """
         Edits a local command matching the specified name.
 
         Parameters
@@ -807,13 +822,13 @@ class SlashClient:
         
         slash_command : SlashCommand
             replacement of the old data
-        '''
+        """
         cmd = self.get_guild_command_named(guild_id, name)
         if cmd is not None:
             await self.edit_guild_command(guild_id, cmd.id, slash_command)
 
     async def delete_global_command_named(self, name: str):
-        '''
+        """
         Deletes a global command matching the specified name.
 
         Parameters
@@ -821,13 +836,13 @@ class SlashClient:
 
         name : str
             the name of the command to delete
-        '''
+        """
         cmd = self.get_global_command_named(name)
         if cmd is not None:
             await self.delete_global_command(cmd.id)
 
     async def delete_guild_command_named(self, guild_id: int, name: str):
-        '''
+        """
         Deletes a local command matching the specified name.
 
         Parameters
@@ -838,7 +853,7 @@ class SlashClient:
 
         name : str
             the name of the command to edit
-        '''
+        """
         cmd = self.get_guild_command_named(guild_id, name)
         if cmd is not None:
             await self.delete_guild_command(guild_id, cmd.id)
@@ -978,6 +993,7 @@ class SlashClient:
         listeners.append((future, check))
         return await asyncio.wait_for(future, timeout)
 
+
     # Adding relevant listeners
     async def _on_socket_response(self, payload):
         if payload.get("t") != "INTERACTION_CREATE":
@@ -1031,6 +1047,9 @@ class SlashClient:
                         else:
                             future.set_result(args)
                         removed.append(i)
+                    else:
+                        # Add on_check_fail in the future
+                        pass
 
             if len(removed) == len(listeners):
                 self._listeners.pop(event)
@@ -1039,9 +1058,6 @@ class SlashClient:
                     del listeners[idx]
 
     async def _activate_event(self, event_name, *args, **kwargs):
-        '''
-        # Don't use it
-        '''
         await self._toggle_listeners(event_name, *args, **kwargs)
         if event_name != "ready":
             self.client.dispatch(event_name, *args, **kwargs)
@@ -1067,8 +1083,12 @@ class SlashClient:
                 except Exception as err:
                     await self._activate_event('slash_command_error', inter, err)
         elif _type == 3:
-            inter = ButtonInteraction(self.client, payload)
-            await self._activate_event('button_click', inter)
+            inter = MessageInteraction(self.client, payload)
+            if inter.component.type == ComponentType.Button:
+                await self._activate_event('button_click', inter)
+            elif inter.component.type == ComponentType.SelectMenu:
+                # FIXME: naming might be different
+                await self._activate_event('select_menu_click', inter)
     
     # Aliases
     register_global_command = register_global_slash_command
